"""Support for LK3 binary sensors."""
import logging
from typing import Callable, List, Union

from homeassistant.config_entries import ConfigEntry
from homeassistant.helpers.entity import Entity
from homeassistant.helpers.typing import HomeAssistantType

from . import LK3DeviceEntity
from .const import DATA_LK3_CLIENT, DOMAIN
from .lk3api import LK3Reader

_LOGGER = logging.getLogger(__name__)

SENSOR_LIST = [
    {'name': 'INPD1', 'key': 'inpd1'},
    {'name': 'INPD2', 'key': 'inpd2'},
    {'name': 'INPD3', 'key': 'inpd3'},
    {'name': 'INPD4', 'key': 'inpd4'},
]


async def async_setup_entry(
        hass: HomeAssistantType,
        entry: ConfigEntry,
        async_add_enities: Callable[[List[Entity], bool], None],
) -> None:
    """Set up LK3 binary sensor based on a config entry."""
    reader: LK3Reader = hass.data[DOMAIN][entry.entry_id][DATA_LK3_CLIENT]

    sensors = [
        LK3BinarySensor(
            entry.entry_id,
            reader,
            item['name'],
            None,
            item['key'],
        ) for item in SENSOR_LIST
    ]
    async_add_enities(sensors, True)


class LK3BinarySensor(LK3DeviceEntity):
    """LK3BinarySensor."""

    def __init__(
            self,
            entry_id: str,
            reader: LK3Reader,
            name: str,
            icon: str,
            key: str,
    ) -> None:
        """Initialize LK3 sensor."""
        self._state = False
        self._key = key
        super().__init__(entry_id, reader, name, icon)

    @property
    def is_on(self) -> bool:
        """Return true if sensor is on."""
        return self._state

    @property
    def unique_id(self) -> str:
        """Return the unique ID for this sensor."""
        return f"{self.reader.device.info.mac_address}_{self._key}"

    @property
    def state(self) -> Union[None, str, int, float]:
        """Return the state of the sensor."""
        return self._state

    @property
    def entity_registry_enabled_default(self) -> bool:
        """Return the default state of registered entity."""
        return False

    async def _lk3_update(self) -> None:
        """Update LK3 entity."""
        self._state = self.reader.device.state[self._key]
