"""Asynchronous Python client for LK3."""
import asyncio
import json
import socket
from typing import Any, Mapping, Optional

import aiohttp
import async_timeout

from .__version__ import __version__
from .exceptions import (LK3AuthenticationError, LK3ConnectionError, LK3Error,
                         LK3VersionError)
from .models import Device


class LK3Reader():
    """Main class for handling connections with LK3."""
    device: Optional[Device] = None

    def __init__(
            self,
            host: str,
            username: str = None,
            password: str = None,
            port: int = 80,
            request_timeout: int = 5,
            session: aiohttp.client.ClientSession = None,
    ) -> None:
        self.host = host
        self.username = username
        self.password = password
        self.port = port
        self.request_timeout = request_timeout
        # aiohttp session
        self._session = session
        self._close_session = False

    async def _request(
            self,
            uri: str,
            method: str = 'GET',
            params: Optional[Mapping[str, str]] = None,
    ) -> Any:
        """Handle a request to a LK3 device."""
        url = f'http://{self.host}:{self.port}{uri}'
        auth = None
        if self.username and self.password:
            auth = aiohttp.BasicAuth(self.username, self.password)
        if self._session is None:
            self._session = aiohttp.ClientSession()
            self._close_session = True
        try:
            with async_timeout.timeout(self.request_timeout):
                response = await self._session.request(
                    method,
                    url,
                    auth=auth,
                    params=params,
                )
        except asyncio.TimeoutError as exc:
            raise LK3ConnectionError('Timeout occured while connecting to LK3 device.') from exc
        except (aiohttp.ClientError, socket.gaierror) as exc:
            raise LK3ConnectionError('Error occured while communicating with LK3 device') from exc
        content_type = response.headers.get('Content-Type', '')
        if (response.status // 100) in [4, 5]:
            contents = await response.read()
            response.close()
            if content_type == 'application/json':
                error_msg = json.loads(contents.decode('utf8'))
            else:
                error_msg = {'message': contents.decode('utf8')}
            if response.status == 401:
                raise LK3AuthenticationError(response.status, error_msg)
            elif response.status == 404:
                raise LK3VersionError(response.status, error_msg)
            raise LK3ConnectionError(response.status, error_msg)
        if 'application/json' in content_type:
            return await response.json()
        return await response.text()

    async def update(self) -> Optional[Device]:
        """Get all information about the device."""
        data = {
            'info': {},
            'state': {},
        }
        try:
            data['state'].update(await self._request('/json/all.json'))
            data['info'].update(data['state'])
            try:
                self.device = Device.from_dict(data)
            except (ValueError, IndexError) as exc:
                raise LK3Error from exc
        except LK3Error as exc:
            self.device = None
            raise exc
        return self.device

    async def output(self, key: str, on: bool = False) -> None:
        """Send command to toggle output."""
        await self._request('/outs.cgi', params={'out': key})

    async def pwm_output(self, key: str, on: bool = False) -> None:
        """Send command to toggle pwm output."""
        await self._request('/outs.cgi', params={'pwm': key})

    async def close(self) -> None:
        """Close open client session."""
        if self._session and self._close_session:
            await self._session.close()

    async def __aenter__(self):
        """Async enter."""
        return self

    async def __aexit__(self, *exc_info):
        """Async exit."""
        await self.close()
